@@ -32,11 +32,9 @@ from libra.common.api.lbaas import HealthMonitor
 from libra.common.exc import ExhaustedError
 from libra.api.model.validators import LBPut, LBPost, LBResp, LBVipResp
 from libra.api.model.validators import LBRespNode
-from libra.common.api.gearman_client import submit_job
+from libra.common.api.gearman_client import submit_job, submit_vip_job
 from libra.api.acl import get_limited_to_project
 from libra.api.library.exp import OverLimit, IPOutOfRange, NotFound
-from libra.api.library.exp import ImmutableEntity, ImmutableStates
-from libra.api.library.exp import ImmutableStatesNoError
 from libra.api.library.ip_filter import ipfilter
 from pecan import conf
 from wsme import types as wtypes
@@ -104,7 +102,7 @@ class LoadBalancersController(RestController):
                     LoadBalancer.updated, LoadBalancer.errmsg,
                     Vip.id.label('vipid'), Vip.ip
                 ).join(LoadBalancer.devices).\
-                    outerjoin(Device.vip).\
+                    join(Device.vip).\
                     filter(LoadBalancer.tenantid == tenant_id).\
                     filter(LoadBalancer.id == self.lbid).\
                     first()
@@ -117,28 +115,20 @@ class LoadBalancersController(RestController):
                 load_balancers['nodeCount'] = session.query(Node).\
                     filter(Node.lbid == load_balancers['id']).count()
 
-                if load_balancers['vipid']:
-                    load_balancers['virtualIps'] = [{
-                        "id": load_balancers['vipid'],
-                        "type": "PUBLIC",
-                        "ipVersion": "IPV4",
-                        "address": str(ipaddress.IPv4Address(
-                            load_balancers['ip']
-                        )),
-                    }]
-                    del(load_balancers['ip'])
-                    del(load_balancers['vipid'])
-                else:
-                    # We are still assigning a VIP
-                    load_balancers['virtualIps'] = [{
-                        "id": None,
-                        "type": "ASSIGNING",
-                        "ipVersion": "IPV4",
-                        "address": None
-                    }]
-                    del(load_balancers['vipid'])
+                load_balancers['virtualIps'] = [{
+                    "id": load_balancers['vipid'],
+                    "type": "PUBLIC",
+                    "ipVersion": "IPV4",
+                    "address": str(ipaddress.IPv4Address(
+                        load_balancers['ip']
+                    )),
+                }]
+                del(load_balancers['ip'])
+                del(load_balancers['vipid'])
+
                 nodes = session.query(
-                    Node.id, Node.address, Node.port, Node.status, Node.enabled
+                    Node.id, Node.address, Node.port, Node.status,
+                    Node.enabled, Node.weight
                 ).join(LoadBalancer.nodes).\
                     filter(LoadBalancer.tenantid == tenant_id).\
                     filter(LoadBalancer.id == self.lbid).\
@@ -162,6 +152,8 @@ class LoadBalancersController(RestController):
                     del node['enabled']
                     node['port'] = str(node['port'])
                     node['id'] = str(node['id'])
+                    if node['weight'] == 1:
+                        del node['weight']
                     load_balancers['nodes'].append(node)
 
             session.rollback()
@@ -192,6 +184,16 @@ class LoadBalancersController(RestController):
             raise ClientSideError(
                 'At least one backend node needs to be supplied'
             )
+
+        # When the load balancer is used for Galera, we need to do some
+        # sanity checking of the nodes to make sure 1 and only 1 node is
+        # defined as the primary node.
+        if body.protocol and body.protocol.lower() == 'galera':
+            is_galera = True
+        else:
+            is_galera = False
+        num_galera_primary_nodes = 0
+
         for node in body.nodes:
             if node.address == Unset:
                 raise ClientSideError(
@@ -206,6 +208,7 @@ class LoadBalancersController(RestController):
                     'Node {0} port number {1} is invalid'
                     .format(node.address, node.port)
                 )
+
             try:
                 node.address = ipfilter(node.address, conf.ip_filters)
             except IPOutOfRange:
@@ -217,6 +220,31 @@ class LoadBalancersController(RestController):
                 raise ClientSideError(
                     'IP Address {0} not valid'.format(node.address)
                 )
+
+            if node.weight != Unset:
+                try:
+                    weight = int(node.weight)
+                except ValueError:
+                    raise ClientSideError(
+                        'Node weight must be an integer'
+                    )
+                if weight < 1 or weight > 256:
+                    raise ClientSideError(
+                        'Node weight must be between 1 and 256'
+                    )
+
+            is_backup = False
+            if node.backup != Unset and node.backup == 'TRUE':
+                is_backup = True
+            if is_galera and not is_backup:
+                num_galera_primary_nodes += 1
+
+        # Galera sanity checks
+        if is_galera and num_galera_primary_nodes != 1:
+            raise ClientSideError(
+                'Galera load balancer must have exactly one primary node'
+            )
+
         with db_session() as session:
             lblimit = session.query(Limits.value).\
                 filter(Limits.name == 'maxLoadBalancers').scalar()
@@ -253,8 +281,13 @@ class LoadBalancersController(RestController):
             lb = LoadBalancer()
             lb.tenantid = tenant_id
             lb.name = body.name
-            if body.protocol and body.protocol.lower() == 'tcp':
-                lb.protocol = 'TCP'
+            if body.protocol:
+                if body.protocol.lower() in ('tcp', 'http', 'galera'):
+                    lb.protocol = body.protocol.upper()
+                else:
+                    raise ClientSideError(
+                        'Invalid protocol %s' % body.protocol
+                    )
             else:
                 lb.protocol = 'HTTP'
 
@@ -267,8 +300,10 @@ class LoadBalancersController(RestController):
             else:
                 if lb.protocol == 'HTTP':
                     lb.port = 80
-                else:
+                elif lb.protocol == 'TCP':
                     lb.port = 443
+                elif lb.protocol == 'GALERA':
+                    lb.port = 3306
 
             lb.status = 'BUILD'
             lb.created = None
@@ -282,22 +317,26 @@ class LoadBalancersController(RestController):
                         session.query(loadbalancers_devices.c.device)
                     )).\
                     filter(Device.status == "OFFLINE").\
-                    filter(Device.pingCount == 0).\
                     with_lockmode('update').\
                     first()
+                NULL = None  # For pep8
+                vip = session.query(Vip).\
+                    filter(Vip.device == NULL).\
+                    with_lockmode('update').\
+                    first()
+                vip.device = device.id
                 if device is None:
                     session.rollback()
                     raise ExhaustedError('No devices available')
-
-                vip = None
+                if vip is None:
+                    session.rollback()
+                    raise ExhaustedError('No virtual IPs available')
+                vip.device = device.id
+                submit_vip_job(
+                    'ASSIGN', device.name, str(ipaddress.IPv4Address(vip.ip))
+                )
             else:
                 virtual_id = body.virtualIps[0].id
-                # Make sure virtual ID is actually an int
-                try:
-                    virtual_id = int(virtual_id)
-                except:
-                    session.rollback()
-                    raise NotFound('Invalid virtual IP provided')
                 # This is an additional load balancer
                 device = session.query(
                     Device
@@ -305,6 +344,13 @@ class LoadBalancersController(RestController):
                     filter(Vip.id == virtual_id).\
                     first()
 
+                if device.status == 'ERROR':
+                    session.rollback()
+                    raise ClientSideError(
+                        'Cannot add a Load Balancer to a device'
+                        ' in an ERROR state'
+                    )
+
                 old_lb = session.query(
                     LoadBalancer
                 ).join(LoadBalancer.devices).\
@@ -312,15 +358,6 @@ class LoadBalancersController(RestController):
                     filter(LoadBalancer.tenantid == tenant_id).\
                     filter(Vip.id == virtual_id).\
                     first()
-
-                if old_lb.status in ImmutableStates:
-                    session.rollback()
-                    raise ImmutableEntity(
-                        'Existing Load Balancer on VIP in a non-ACTIVE state'
-                        ', current state: {0}'
-                        .format(old_lb.status)
-                    )
-
                 vip = session.query(Vip).\
                     filter(Vip.device == device.id).\
                     first()
@@ -361,17 +398,25 @@ class LoadBalancersController(RestController):
                 else:
                     enabled = 1
                     node_status = 'ONLINE'
+
+                if node.backup == 'TRUE':
+                    backup = 1
+                else:
+                    backup = 0
+
+                weight = 1
+                if node.weight != Unset:
+                    weight = node.weight
                 out_node = Node(
                     lbid=lb.id, port=node.port, address=node.address,
-                    enabled=enabled, status=node_status, weight=1
+                    enabled=enabled, status=node_status,
+                    weight=weight, backup=backup
                 )
                 session.add(out_node)
 
             # now save the loadbalancer_id to the device and switch its status
-            # to build so the monitoring does not trigger early.
-            # The gearman message code will switch to ONLINE once we know
-            # everything is good
-            device.status = "BUILD"
+            # to online
+            device.status = "ONLINE"
             session.flush()
 
             return_data = LBResp()
@@ -383,29 +428,30 @@ class LoadBalancersController(RestController):
             return_data.status = lb.status
             return_data.created = lb.created
             return_data.updated = lb.updated
-            if vip:
-                vip_resp = LBVipResp(
-                    address=str(ipaddress.IPv4Address(vip.ip)),
-                    id=str(vip.id), type='PUBLIC', ipVersion='IPV4'
-                )
-            else:
-                vip_resp = LBVipResp(
-                    address=None, id=None, type='ASSIGNING', ipVersion='IPV4'
-                )
+            vip_resp = LBVipResp(
+                address=str(ipaddress.IPv4Address(vip.ip)),
+                id=str(vip.id), type='PUBLIC', ipVersion='IPV4'
+            )
             return_data.virtualIps = [vip_resp]
             return_data.nodes = []
             for node in body.nodes:
-                out_node = LBRespNode(
-                    port=str(node.port), address=node.address,
-                    condition=node.condition
-                )
+                if node.weight != Unset and node.weight != 1:
+                    out_node = LBRespNode(
+                        port=str(node.port), address=node.address,
+                        condition=node.condition, weight=weight
+                    )
+                else:
+                    out_node = LBRespNode(
+                        port=str(node.port), address=node.address,
+                        condition=node.condition
+                    )
+
                 return_data.nodes.append(out_node)
             session.commit()
             # trigger gearman client to create new lb
             submit_job(
                 'UPDATE', device.name, device.id, lb.id
             )
-
             return return_data
 
     @wsme_pecan.wsexpose(None, body=LBPut, status_code=202)
@@ -425,14 +471,6 @@ class LoadBalancersController(RestController):
                 session.rollback()
                 raise NotFound('Load Balancer ID is not valid')
 
-            if lb.status in ImmutableStates:
-                session.rollback()
-                raise ImmutableEntity(
-                    'Cannot modify a Load Balancer in a non-ACTIVE state'
-                    ', current state: {0}'
-                    .format(lb.status)
-                )
-
             if body.name != Unset:
                 namelimit = session.query(Limits.value).\
                     filter(Limits.name == 'maxLoadBalancerNameLength').scalar()
@@ -453,6 +491,12 @@ class LoadBalancersController(RestController):
                 filter(LoadBalancer.id == self.lbid).\
                 first()
 
+            if device.status == 'ERROR':
+                session.rollback()
+                raise ClientSideError(
+                    'Cannot update a Load Balancer in an ERROR state'
+                )
+
             session.commit()
             submit_job(
                 'UPDATE', device.name, device.id, lb.id
@@ -486,14 +530,6 @@ class LoadBalancersController(RestController):
             if lb is None:
                 session.rollback()
                 raise NotFound("Load Balancer ID is not valid")
-            # So we can delete ERROR, but not other Immutable states
-            if lb.status in ImmutableStatesNoError:
-                session.rollback()
-                raise ImmutableEntity(
-                    'Cannot delete a Load Balancer in a non-ACTIVE state'
-                    ', current state: {0}'
-                    .format(lb.status)
-                )
             lb.status = 'PENDING_DELETE'
             device = session.query(
                 Device.id, Device.name
