@@ -12,7 +12,6 @@
 # License for the specific language governing permissions and limitations
 # under the License.
 
-import socket
 from libra import __version__ as libra_version
 from libra import __release__ as libra_release
 from libra.common.exc import DeletedStateError
@@ -35,11 +34,10 @@ class LBaaSController(object):
     OBJ_STORE_ENDPOINT_FIELD = 'hpcs_object_store_endpoint'
     OBJ_STORE_TOKEN_FIELD = 'hpcs_object_store_token'
 
-    def __init__(self, logger, driver, json_msg, gearman):
+    def __init__(self, logger, driver, json_msg):
         self.logger = logger
         self.driver = driver
         self.msg = json_msg
-        self.gearman = gearman
 
     def run(self):
         """
@@ -68,8 +66,6 @@ class LBaaSController(object):
                 return self._action_archive()
             elif action == 'STATS':
                 return self._action_stats()
-            elif action == 'DIAGNOSTICS':
-                return self._action_diagnostic()
             else:
                 self.logger.error("Invalid `%s` value: %s" %
                                   (self.ACTION_FIELD, action))
@@ -81,54 +77,6 @@ class LBaaSController(object):
             self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
             return self.msg
 
-    def _action_diagnostic(self):
-        """
-        Returns the results of a diagnostic run
-
-        This message is used to see if the worker that was built will actually
-        function as a load balancer
-        """
-        # Gearman test
-        self.msg['gearman'] = []
-        for host_port in self.gearman:
-            host, port = host_port.split(':')
-            try:
-                self._check_host(host, int(port))
-            except:
-                self.msg['gearman'].append(
-                    {'host': host, 'status': self.RESPONSE_FAILURE}
-                )
-            else:
-                self.msg['gearman'].append(
-                    {'host': host, 'status': self.RESPONSE_SUCCESS}
-                )
-        # Outgoing network test
-        try:
-            # TODO: make this configurable
-            self._check_host('google.com', 80)
-        except:
-            self.msg['network'] = self.RESPONSE_FAILURE
-        else:
-            self.msg['network'] = self.RESPONSE_SUCCESS
-
-        self.msg[self.RESPONSE_FIELD] = self.RESPONSE_SUCCESS
-        return self.msg
-
-    def _check_host(self, ip, port):
-        # TCP connect check to see if floating IP was assigned correctly
-        sock = socket.socket()
-        sock.settimeout(5)
-        try:
-            sock.connect((ip, port))
-            sock.close()
-            return True
-        except socket.error:
-            self.logger.error(
-                "TCP connect error to gearman server {0}"
-                .format(ip)
-            )
-            raise
-
     def _action_discover(self):
         """
         Return service discovery information.
@@ -254,7 +202,11 @@ class LBaaSController(object):
                     return self.msg
 
             for lb_node in current_lb['nodes']:
-                port, address, node_id, weight = None, None, None, None
+                port = None
+                address = None
+                node_id = None
+                weight = None
+                backup = False
 
                 if 'port' in lb_node:
                     port = lb_node['port']
@@ -274,21 +226,30 @@ class LBaaSController(object):
                 if 'weight' in lb_node:
                     weight = lb_node['weight']
 
+                if 'backup' in lb_node and lb_node['backup'].lower() == 'true':
+                    backup = True
+
                 try:
                     self.driver.add_server(current_lb['protocol'],
                                            node_id,
                                            address,
                                            port,
-                                           weight)
+                                           weight,
+                                           backup)
                 except NotImplementedError:
-                    self.logger.error(
-                        "Selected driver does not support adding a server."
-                    )
                     lb_node['condition'] = self.NODE_ERR
+                    error = "Selected driver does not support adding a server"
+                    self.logger.error(error)
+                    self.msg[self.ERROR_FIELD] = error
+                    self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
+                    return self.msg
                 except Exception as e:
-                    self.logger.error("Failure trying adding server: %s, %s" %
-                                      (e.__class__, e))
                     lb_node['condition'] = self.NODE_ERR
+                    error = "Failure adding server %s: %s" % (node_id, e)
+                    self.logger.error(error)
+                    self.msg[self.ERROR_FIELD] = error
+                    self.msg[self.RESPONSE_FIELD] = self.RESPONSE_FAILURE
+                    return self.msg
                 else:
                     self.logger.debug("Added server: %s:%s" % (address, port))
                     lb_node['condition'] = self.NODE_OK
